/*
  additive Farbmischung

  created 2019
  by Marco Janßen

  License:
  Attribution-NonCommercial-ShareAlike 4.0 International
  (CC BY-NC-SA 4.0)
  https://creativecommons.org/licenses/by-nc-sa/4.0/
*/


p5.disableFriendlyErrors = true; // disables FES


const max_value_slider = 12;
let wheel;
let mode;
let c_active;
let button_mode;
let button_red;
let button_green;
let button_blue;
let button_mode_state;
let	button_red_state;
let	button_green_state;
let	button_blue_state;


function setup() {
  createCanvas(windowWidth, windowHeight);
  angleMode(DEGREES);
	
	mode = 0;
	c_active = 3;
	
	button_mode_state = 1;
	button_red_state = 1;
	button_green_state = 1;
	button_blue_state = 1;
	resizeLayout();
}


function draw() {
	background(0);
	
	switch(mode) {
		case 0:
		wheel.update();
		wheel.render();
		break;
		
		case 1:
		renderLines();
		break;

		case 2:
		renderLightCircles();
		break;
	}
	
	slider.render();
	button_mode.render();
	button_red.render();
	button_green.render();
	button_blue.render();
}


function renderLightCircles() {
	let sizeCircle = min(width, height) * 0.45;
	let distCircles = (max_value_slider - slider.value) * sizeCircle * 0.05;

	push();
	blendMode(ADD);
	noStroke();
	if(button_red.getState()){
		fill(255, 0, 0);
		ellipse(width * 0.5, height * 0.5 - distCircles * 0.707, sizeCircle);
	}

	if(button_green.getState()){
	fill(0, 255, 0);
	ellipse(width * 0.5 - distCircles, height * 0.5 + distCircles, sizeCircle);
	}

	if(button_blue.getState()) {
		fill(0, 0, 255);
		ellipse(width * 0.5 + distCircles, height * 0.5 + distCircles, sizeCircle);
	}
	pop();

}


function renderLines() {
	push();
	strokeWeight(1);
	let d = pixelDensity();
	let value = max_value_slider - slider.value + 1;
	for (let i = 0; i < width * d; i++) {
		let c_type = floor(i / value) % c_active;
		switch (c_type) {
			case 0:
				if(button_red.getState())
					stroke(255, 0, 0);
				else if(button_green.getState())
					stroke(0, 255, 0);
				else if(button_blue.getState())
					stroke(0, 0, 255);
				break;
			case 1:
				if(button_green.getState())
					stroke(0, 255, 0);
				else if(button_blue.getState())
					stroke(0, 0, 255);
				else if(button_red.getState())
					stroke(255, 0, 0);
				
				if(!button_red.getState() && button_green.getState() && button_blue.getState())
					stroke(0, 0, 255);
				break;
			case 2:
				if(button_blue.getState())
					stroke(0, 0, 255);
				else if(button_red.getState())
					stroke(255, 0, 0);
				else if(button_green.getState())
					stroke(0, 255, 0);
				break;
		}
		if(c_active == 0)
			stroke(0, 0, 0);
 		rect(i / d, height * 0.1, value / d, height * 0.8);
	}
	pop();
}


function windowResized() {
  resizeCanvas(windowWidth, windowHeight);
	resizeLayout();
} 


function resizeLayout() {
	let temp_size = height * 0.05;
	let temp_width = width * 0.9;
	
	button_mode = new Button(width * 0.05, height * 0.025, width * (0.9 - 0.15) * 0.25, height * 0.05, color(40), color(80), color(255),button_mode_state, 1, "Modus");
	
	button_red = new Button(width * 0.05 * 2 + width * (0.9 - 0.15) * 0.25 * 1, 
	height * 0.025, width * (0.9 - 0.15) * 0.25, height * 0.05, color(80, 0, 0), color(255, 0, 0), color(255), button_red_state, 1, " ");
	
	button_green = new Button(width * 0.05 * 3 + width * (0.9 - 0.15) * 0.25 * 2, 
	height * 0.025, width * (0.9 - 0.15) * 0.25, height * 0.05, color(0, 80, 0), color(0, 255, 0), color(255), button_green_state, 1, " ");
	
	button_blue = new Button(width * 0.05 * 4 + width * (0.9 - 0.15) * 0.25 * 3, 
	height * 0.025, width * (0.9 - 0.15) * 0.25, height * 0.05, color(0, 0, 80), color(0, 0, 255), color(255), button_blue_state, 1, " ");
	
	slider = {
  	pressed : false,
		x : width * 0.05,
		y : height * 0.925,
		x_offset : temp_size * 0.5,
		width : temp_width,
		height : temp_size,
		intervall : (temp_width - temp_size) / (max_value_slider - 1),
		value : round(max_value_slider * 0.75),
		
		render : function() {
			push();
			rectMode(CORNER);
			fill(40);
			stroke(255);
    	this.pressed ? strokeWeight(2) : strokeWeight(1); 
      rect(this.x, this.y + this.height * 0.35, this.width, this.height * 0.3, this.height);
      fill(80);
    	ellipse(this.x + (this.value - 1) * this.intervall + this.x_offset, this.y + this.height * 0.5, this.height);
			pop();
		},
		
		updateValue : function() {
			if (this.pressed)
  		this.value = constrain(round(map(mouseX, this.x + this.x_offset, this.x + this.width - this.x_offset, 1, max_value_slider)), 1, max_value_slider);
		},
		
		checkPressed : function() {
			if (mouseX > this.x && mouseX < this.x + this.width && mouseY > this.y && mouseY < this.y + this.height)
				this.pressed = true;
			else 
				this.pressed = false;
		}
	}
	
	let rad_temp;
	width < height ? rad_temp = width * 0.45 : rad_temp = height * 0.4;
    
	wheel = {
		x : width * 0.5,
		y : height * 0.5,
		angle : 0,
		vel : 0,
		acc : 0,
		damp : 0.997,
		force : 0.014,
		rad : rad_temp,
		touched : false,
		max_vel : 40,
		
		render : function() {
			push();
			fill(255);
			stroke(255);
			if (this.touched)
				strokeWeight(4);
			else
				strokeWeight(2);
			translate(this.x, this.y);
			rotate(this.angle);
			ellipse(0, 0, this.rad * 2);
			noStroke();

			for (let i = 0; i < slider.value; i++) {
				push();
				let part_width = 360 / slider.value;
				let part_angle = part_width * i;
				rotate(part_angle);

				let part_color;
				let c_type = i % c_active;
				switch (c_type) {
					case 0:
						if(button_red.getState())
							part_color = color(255, 0, 0);
						else if(button_green.getState())
							part_color = color(0, 255, 0);
						else if(button_blue.getState())
							part_color = color(0, 0, 255);
						break;
					case 1:
						if(button_green.getState())
							part_color = color(0, 255, 0);
						else if(button_blue.getState())
							part_color = color(0, 0, 255);
						else if(button_red.getState())
							part_color = color(255, 0, 0);
			
						if(!button_red.getState() && button_green.getState() && button_blue.getState())
							part_color = color(0, 0, 255);
						break;
					case 2:
						if(button_blue.getState())
							part_color = color(0, 0, 255);
						else if(button_red.getState())
							part_color = color(255, 0, 0);
						else if(button_green.getState())
							part_color = color(0, 255, 0);
						break;
				}
				if(c_active == 0)
					part_color = color(0, 0, 0);
				fill(part_color);
				arc(0, 0, this.rad * 2, this.rad * 2, 0 - 90, part_width - 90, PIE);
				pop();
			}
			pop();
		},
		
		checkPressed : function() {
			if (dist(mouseX, mouseY, this.x, this.y) < this.rad) 
				this.touched = true;
		},

		accelerate : function() {
			if (dist(mouseX, mouseY, this.x, this.y) < this.rad && this.touched) {
				let half_width = this.x;
				let half_height = this.y;

				let prev_mouseX = pmouseX;
				let prev_mouseY = pmouseY;
				let current_mouseX = mouseX;
				let current_mouseY = mouseY;

				if (current_mouseX < half_width && current_mouseY < half_height) {
					current_mouseY *= -1;
					prev_mouseY *= -1;
				}

				if (current_mouseX < half_width && current_mouseY > half_height) {
					current_mouseX *= -1;
					prev_mouseX *= -1;
				}

				if (current_mouseX < half_width && current_mouseY > half_height) {
					current_mouseY *= -1;
					prev_mouseY *= -1;
				}

				if (current_mouseX > half_width && current_mouseY > half_height) {
					current_mouseX *= -1;
					prev_mouseX *= -1;
				}

				this.acc = (current_mouseX - prev_mouseX + current_mouseY - prev_mouseY) * this.force;
				this.vel < this.max_vel ? this.vel += this.acc : this.vel = this.max_vel;
			}
		},
		
		update : function() {
			this.angle += this.vel;
			this.angle %= 360;
			this.angle < 0 ? this.angle = 360 : this.angle = this.angle;

			if (this.vel < 0) {
				this.vel *= this.damp;
				if (this.vel > -0.05)
					this.vel = 0;
			} else if (this.vel > 0) {
				this.vel *= this.damp;
				if (this.vel < 0.05)
					this.vel = 0;
  		}
		}
	}
}


function dragged() {
	wheel.accelerate();
	slider.updateValue();
}


function touchMoved() {
	dragged();
	
	if(mouseY < height)
	 	return false;
}


function mouseDragged() {
  dragged();
}


function mousePressed() {
	wheel.checkPressed();
	slider.checkPressed();
  slider.updateValue();
	if(button_mode.checkPressed()) {
		mode++;
		mode %= 3;
		button_mode_state = button_mode.state;
		switch(mode) {
			case 0:
			slider.value = round(max_value_slider * 0.75);
			break;
			case 1:
			slider.value = round(max_value_slider * 0.25);
			break;
			case 2:
			slider.value = 1;
			break;
		}

	}
	if(button_red.checkPressed()) {
		button_red.getState() ? c_active++ : c_active--;
		button_red_state = button_red.state
	}
	if(button_green.checkPressed()) {
		button_green.getState() ? c_active++ : c_active--;
		button_green_state = button_green.state
	}
	if(button_blue.checkPressed()) {
		button_blue.getState() ? c_active++ : c_active--;
		button_blue_state = button_blue.state
	}

	return false;
}


function mouseReleased() {
	wheel.touched = false;
  slider.pressed = false;
	button_mode.pressedOff();
	button_red.pressedOff();
	button_green.pressedOff();
	button_blue.pressedOff();
}



class Button {
	constructor (_x, _y, _width, _height, _c0, _c1, _c_stroke, _state, _max_state, _label) {
		this.x = _x;
		this.y = _y;
		this.width = _width;
		this.height = _height;
		this.c0 = _c0;
		this.c1 = _c1;
		this.c_current = _c1;
		this.c_stroke = _c_stroke;
		this.pressed = false;
		this.state = _state;
		this.max_state = _max_state;
		this.label = _label;
	}
	
	checkPressed() {
		if (mouseX > this.x && mouseX < this.x + this.width && mouseY > this.y && mouseY < this.y + this.height){
			this.pressed = true;
			this.state++;
			this.state %= (this.max_state + 1);
			return true;
		} else {
			this.pressed = false;
			return false;
		}
	}

	pressedOff() {
		this.pressed = false;
	}
	
	getState() {
		return this.state;
	}
	
	render() {
		push();
		switch(this.state) {
			case 0:
			this.c_current = this.c0;
			break;
			case 1:
			this.c_current = this.c1;
			break;
		}
		
		if(this.pressed) {
			strokeWeight(4);
			this.c_current.setAlpha(150);
		} else {
			strokeWeight(2);
			this.c_current.setAlpha(255);
		}
		
		stroke(this.c_stroke);
		fill(255);
		rect(this.x, this.y, this.width, this.height, this.height * 0.1);
		fill(this.c_current)
		rect(this.x, this.y, this.width, this.height, this.height * 0.1);
		
		fill(this.c_stroke);
		strokeWeight(1);
		textAlign(CENTER, CENTER);
		textSize(this.height * 0.5);
		textFont('Courier');
		text(this.label, this.x + this.width * 0.5, this.y + this.height * 0.5)
		
		pop();
	}
}